using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using EnvDTE80;
using EnvDTE;

namespace Com.Benday.VisualStudio.Addins.SolutionSearch
{
    public partial class AddinDebugger : Form
    {
        private DTE2 m_dte;
        private Dictionary<string, Command> m_commands;
        private AddIn m_addInInstance;

        public AddinDebugger()
        {
            InitializeComponent();
        }

        public void Initialize(DTE2 dte, AddIn addInInstance)
        {
            try
            {
                m_dte = dte;
                m_addInInstance = addInInstance;
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }            
        }

        private void m_btnSearch_Click(object sender, EventArgs e)
        {
            try
            {
                List<string> matches = new List<string>();

                m_listCommands.DataSource = null;
                m_listCommands.Items.Clear();
                
                string search = m_textCommandName.Text;

                foreach (string key in m_commands.Keys)
                {
                    if (key.Contains(search) == true)
                    {
                        matches.Add(key);
                    }
                }

                m_listCommands.DataSource = matches;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private void m_btnPopulateCommands_Click(object sender, EventArgs e)
        {
            try
            {
                m_status.Text = "Populating...";

                m_commands = new Dictionary<string, Command>();

                foreach (Command cmd in m_dte.Commands)
                {
                    if (m_commands.ContainsKey(cmd.Name) == false)
                    {
                        m_commands.Add(cmd.Name, cmd);
                    }
                }

                m_status.Text = "Populated.";

            }
            catch (Exception ex)
            {
                m_status.Text = "Error";

                MessageBox.Show(ex.ToString());
            }
        }

        private void m_btnCreateCommand_Click(object sender, EventArgs e)
        {
            try
            {
                m_status.Text = "Creating...";
                m_textErrorMessage.Text = String.Empty;

                object[] contextGUIDS = new object[] { };
                
                Commands2 commands = (Commands2)m_dte.Commands;

                Command command = commands.AddNamedCommand2(
                    m_addInInstance, m_textCommandNameNew.Text, m_textButtonText.Text,
                    "Executes the command for BendayVisualStudioSearchAddIn",
                    true, 59,
                    ref contextGUIDS,
                    (int)vsCommandStatus.vsCommandStatusSupported + (int)vsCommandStatus.vsCommandStatusEnabled,
                    (int)vsCommandStyle.vsCommandStyleText,
                    vsCommandControlType.vsCommandControlTypeButton);

                m_status.Text = "Created";

            }
            catch (Exception ex)
            {
                m_status.Text = "Error";

                // MessageBox.Show(ex.ToString());
                m_textErrorMessage.Text = ex.ToString();
            }
        }

        private void m_btnRemoveCommand_Click(object sender, EventArgs e)
        {
            try
            {
                m_textErrorMessage.Text = String.Empty;

                Command deleteThis = null;

                string search = m_textCommandName.Text;

                foreach (string key in m_commands.Keys)
                {
                    if (key.Contains(search) == true)
                    {
                        if (deleteThis != null)
                        {
                            throw new InvalidOperationException("There was more than one to delete...get more specific.");
                        }

                        deleteThis = m_commands[key];
                    }
                }

                if (deleteThis == null)
                {
                    m_textErrorMessage.Text = "Could not find command to delete (" + search + ")";
                }
                else
                {
                    deleteThis.Delete();
                    m_textErrorMessage.Text = "Deleted.";
                }
            }
            catch (Exception ex)
            {
                m_textErrorMessage.Text = ex.ToString();
            }
        }

        private void m_btnPrintCommandInfo_Click(object sender, EventArgs e)
        {
            //StringBuilder sb = new StringBuilder();

            //try
            //{
                
            //    m_textErrorMessage.Text = String.Empty;

            //    Command debugThis = null;

            //    string search = m_textCommandName.Text;

            //    foreach (string key in m_commands.Keys)
            //    {
            //        if (key.Contains(search) == true)
            //        {
            //            if (debugThis != null)
            //            {
            //                sb.Append("More than one command");
            //                sb.Append(Environment.NewLine);
            //                sb.Append(Environment.NewLine);
            //            }

            //            debugThis = m_commands[key];
            //        }
            //    }

            //    if (debugThis == null)
            //    {
            //        m_textErrorMessage.Text = "Could not find command to delete (" + search + ")";
            //    }
            //    else
            //    {
            //        //debugThis.Guid;
            //        //debugThis.ID;
            //        //debugThis.IsAvailable;
            //        //debugThis.LocalizedName;
            //        //debugThis.Name;
            //        //debugThis.
            //        //m_textErrorMessage.Text = "Deleted.";
            //    }
            //}
            //catch (Exception ex)
            //{
            //    m_textErrorMessage.Text = ex.ToString();
            //}
        }
    }
}