using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE;
using System.IO;

namespace Com.Benday.VisualStudio.Addins.SolutionSearch
{
    public class ProjectItemWrapper
    {
        private TextPoint m_point;
        private string m_nameOriginal;
        private string m_name;
        private string m_fullname;
        private string m_type;

        private ProjectItem m_item;

        public ProjectItemWrapper(CodeElement element, ProjectItem item)
        {
            m_item = item;
            m_nameOriginal = element.Name;
            m_name = element.Name.ToLower();
            m_fullname = element.FullName;

            if (element.Kind == vsCMElement.vsCMElementClass)
            {
                m_type = "CLASS";
                //m_point = element.StartPoint;
            }
            else if (element.Kind == vsCMElement.vsCMElementProperty)
            {
                m_type = "PROPERTY";
                m_point = element.StartPoint;
            }
            else if (element.Kind == vsCMElement.vsCMElementFunction)
            {
                m_type = "FUNCTION";
                m_point = element.StartPoint;
            }
            else if (element.Kind == vsCMElement.vsCMElementInterface)
            {
                m_type = "INTERFACE";
                // m_point = element.StartPoint;
            }
            else 
            {
                m_type = element.Kind.ToString();
            }
        }

        public ProjectItemWrapper(ProjectItem item)
        {
            m_item = item;
            m_name = item.Name.ToLower();
            m_nameOriginal = item.Name;
            m_fullname = item.Name;
            m_type = "PROJECT_ITEM";
        }

        public virtual ProjectItem Item
        {
            get
            {
                return m_item;
            }
            set
            {
                m_item = value;
            }
        }


        public virtual TextPoint TextPoint
        {
            get
            {
                return m_point;
            }
            set
            {
                m_point = value;
            }
        }

        public virtual string Name
        {
            get
            {
                return m_name;
            }
            set
            {
                m_name = value;
            }
        }

        public virtual string FullName
        {
            get
            {
                return m_fullname;
            }
            set
            {
                m_fullname = value;
            }
        }

        public virtual string ItemType
        {
            get
            {
                return m_type;
            }
            set
            {
                m_type = value;
            }
        }

        public string SafeGetPathFromProjectItem()
        {
            return SafeGetPathFromProjectItem(this.Item);
        }

        private string SafeGetPathFromProjectItem(ProjectItem item)
        {
            try
            {
                if (item.FileCount == 0)
                {
                    return "(No path)";
                }
                else
                {
                    Project project = item.ContainingProject;
                    string projectFilename = project.FileName;

                    FileInfo projectFileInfo = new FileInfo(projectFilename);
                    string projectPath = 
                        projectFilename.Substring(0, 
                        projectFilename.Length - projectFileInfo.Name.Length);

                    if (item.get_FileNames(0) != null)
                    {
                        string itemFileName = item.get_FileNames(0);

                        return project.Name + "." + itemFileName.Replace(projectPath, "");
                    }
                    else
                    {
                        return "";
                    }                    
                }
            }
            catch
            {
                return "(Unknown Path)";
            }
        }

        public override string ToString()
        {
            if (this.FullName == null)
            {
                return String.Empty;
            }
            else
            {
                if (this.ItemType == "FUNCTION")
                {
                    return String.Format("{0} - {1}() - {2}", this.ItemType, m_nameOriginal, this.FullName);
                }
                else if (this.ItemType == "PROPERTY" || 
                    this.ItemType == "INTERFACE")
                {
                    return String.Format("{0} - {1} - {2}", this.ItemType, m_nameOriginal, this.FullName);
                }
                else if (this.ItemType == "CLASS" ||
                        this.ItemType == "PROJECT_ITEM")
                {
                    return String.Format("{0} - {1}", this.ItemType, 
                        SafeGetPathFromProjectItem());                     
                }
                else
                {
                    return String.Format("{0} - {1} - {2}", this.ItemType, m_nameOriginal, this.FullName);
                }
            }
        }
    }
}
